set(CMAKE_MODULE_PATH ${CMAKE_MODULE_PATH} "${CMAKE_SOURCE_DIR}/cmake/modules")

######################### utility functions ##########################

# definition of a macro (MACRO) from another one (CACHE_MACRO) that can be already in cache,
# and a default value if CACHE_MACRO exists in cache and not empty
function(xlifeppSetDefaultValue MACRO CACHE_MACRO DEFAULTVALUE)
  set(MACRO_EXISTS -1)
  get_cmake_property(cachelist CACHE_VARIABLES)
  list(FIND cachelist ${CACHE_MACRO} MACRO_EXISTS)
  if (NOT ${MACRO_EXISTS} STREQUAL "-1")
    if (NOT ${${CACHE_MACRO}} STREQUAL " ")
      string(STRIP ${${CACHE_MACRO}} ${CACHE_MACRO})
      set(${MACRO} ${${CACHE_MACRO}} PARENT_SCOPE)
    else()
      set(${MACRO} ${DEFAULTVALUE} PARENT_SCOPE)
    endif()
  else()
    set(${MACRO} ${DEFAULTVALUE} PARENT_SCOPE)
  endif()
  unset(cachelist CACHE)
endfunction()

###################### LAPACK/BLAS management ########################
set(XLIFEPP_ENABLE_LAPACK FALSE)
set(XLIFEPP_ENABLE_BLAS FALSE)

if (XLIFEPP_ENABLE_ARPACK OR XLIFEPP_ENABLE_UMFPACK)
  set(XLIFEPP_ENABLE_LAPACK TRUE)
  set(XLIFEPP_ENABLE_BLAS TRUE)
endif()

##### BLAS -> XLIFEPP_WITH_BLAS or XLIFEPP_WITHOUT_BLAS
##### cache options: XLIFEPP_BLAS_LIB_DIR, XLIFEPP_BLAS_LIB
set(XLIFEPP_BLAS_LIB_DIR " " CACHE PATH "Path to BLAS lib directory")
set(XLIFEPP_BLAS_LIB ${BLAS_LIB_DEFAULT} CACHE FILEPATH "BLAS lib")
set(BLAS XLIFEPP_WITHOUT_BLAS)
if (XLIFEPP_ENABLE_BLAS)
  if (XLIFEPP_BLAS_LIB STREQUAL " ")
    if (NOT XLIFEPP_BLAS_LIB_DIR STREQUAL " ")
      string(STRIP ${XLIFEPP_BLAS_LIB_DIR} XLIFEPP_BLAS_LIB_DIR)
    endif()
    find_blas(${XLIFEPP_BLAS_LIB_DIR} BLAS_FOUND XLIFEPP_BLAS_LIB)
  else()
    string(STRIP ${XLIFEPP_BLAS_LIB} XLIFEPP_BLAS_LIB)
    if (EXISTS ${XLIFEPP_BLAS_LIB})
      set(BLAS_FOUND TRUE)
      get_filename_component(XLIFEPP_BLAS_LIB_DIR ${XLIFEPP_BLAS_LIB} REALPATH)
    else()
      set(BLAS_FOUND FALSE)
    endif()
  endif()
  if (BLAS_FOUND)
    set(BLAS XLIFEPP_WITH_BLAS)
    message(STATUS "BLAS is used")
  else()
    set(BLAS XLIFEPP_WITHOUT_BLAS)
    message(STATUS "BLAS is not found ! It is not used")
  endif()
else()
  message(STATUS "BLAS is not used")
endif()

mark_as_advanced(XLIFEPP_BLAS_LIB)

##### LAPACK -> XLIFEPP_WITH_LAPACK or XLIFEPP_WITHOUT_LAPACK
##### cache option: XLIFEPP_LAPACK_LIB_DIR, XLIFEPP_LAPACK_LIB
set(XLIFEPP_LAPACK_LIB_DIR " " CACHE PATH "Path to LAPACK lib directory")
set(XLIFEPP_LAPACK_LIB ${LAPACK_LIB_DEFAULT} CACHE FILEPATH "LAPACK lib")
set(LAPACK XLIFEPP_WITHOUT_LAPACK)
if (XLIFEPP_ENABLE_LAPACK)
  if (XLIFEPP_LAPACK_LIB STREQUAL " ")
    if (NOT XLIFEPP_LAPACK_LIB_DIR STREQUAL " ")
      string(STRIP ${XLIFEPP_LAPACK_LIB_DIR} XLIFEPP_LAPACK_LIB_DIR)
    endif()
    find_lapack(${XLIFEPP_LAPACK_LIB_DIR} LAPACK_FOUND XLIFEPP_LAPACK_LIB)
  else()
    string(STRIP ${XLIFEPP_LAPACK_LIB} XLIFEPP_LAPACK_LIB)
    if (EXISTS ${XLIFEPP_LAPACK_LIB})
      set(LAPACK_FOUND TRUE)
      get_filename_component(XLIFEPP_LAPACK_LIB_DIR ${XLIFEPP_LAPACK_LIB} REALPATH)
    else()
      set(LAPACK_FOUND FALSE)
    endif()
  endif()
  if (LAPACK_FOUND)
    set(LAPACK XLIFEPP_WITH_LAPACK)
    message(STATUS "LAPACK is used")
  else()
    set(LAPACK XLIFEPP_WITHOUT_LAPACK)
    message(STATUS "LAPACK is not found ! It is not used")
  endif()
else()
  message(STATUS "LAPACK is not used")
endif()

mark_as_advanced(XLIFEPP_LAPACK_LIB)

##### ARPACK -> XLIFEPP_WITH_ARPACK or XLIFEPP_WITHOUT_ARPACK
##### ----------------------- LAPACK and BLAS required --------------------------
##### cache options: XLIFEPP_ARPACK_LIB_DIR, XLIFEPP_ARPACK_LIB, XLIFEPP_ENABLE_ARPACK
##### behavior: if XLIFEPP_ARPACK_LIB_DIR is set, XLIFEPP_ENABLE_ARPACK=ON
#####           if XLIFEPP_ENABLE_ARPACK=OFF, ARPACK_LIB_DIR is removed from cache
# Checking Arpack++ package
set(XLIFEPP_ARPACKPP_INCLUDE_DIR ${CMAKE_SOURCE_DIR}/ext/ARPACK++/include CACHE PATH "Path to Arpack++ include directory")
set(XLIFEPP_ARPACK_LIB_DIR " " CACHE PATH "Path to ARPACK lib directory")
set(XLIFEPP_ARPACK_LIB ${ARPACK_LIB_DEFAULT} CACHE FILEPATH "ARPACK lib")
set(ARPACK XLIFEPP_WITHOUT_ARPACK)
set(AUTOBUILDARPACK FALSE)
if (XLIFEPP_ENABLE_ARPACK)
  set(ARPACK XLIFEPP_WITH_ARPACK)
  set(ARPACK_EXISTS -1)
  include_directories(${XLIFEPP_ARPACKPP_INCLUDE_DIR})
  if(NOT XLIFEPP_ARPACK_LIB_DIR STREQUAL " ")
    string(STRIP ${XLIFEPP_ARPACK_LIB_DIR} XLIFEPP_ARPACK_LIB_DIR)
  endif()
  if(NOT XLIFEPP_ARPACK_LIB STREQUAL " ")
    string(STRIP ${XLIFEPP_ARPACK_LIB} XLIFEPP_ARPACK_LIB)
  endif()
  if (XLIFEPP_SYSTEM_ARPACK)
    if (XLIFEPP_FROM_SRCS)
      if (XLIFEPP_ARPACK_LIB_DIR STREQUAL ${LIBRARY_OUTPUT_PATH})
        set(XLIFEPP_ARPACK_LIB_DIR " " CACHE PATH "Path to ARPACK lib directory" FORCE)
      endif()
      if (XLIFEPP_ARPACK_LIB STREQUAL "${LIBRARY_OUTPUT_PATH}/libarpack.a")
        set(XLIFEPP_ARPACK_LIB " " CACHE FILEPATH "ARPACK lib" FORCE)
      endif()
    else()
      if (XLIFEPP_ARPACK_LIB_DIR STREQUAL "${CMAKE_SOURCE_DIR}/lib")
        set(XLIFEPP_ARPACK_LIB_DIR " " CACHE PATH "Path to ARPACK lib directory" FORCE)
      endif()
      if (XLIFEPP_ARPACK_LIB STREQUAL "${CMAKE_SOURCE_DIR}/lib/libarpack.a")
        set(XLIFEPP_ARPACK_LIB " " CACHE FILEPATH "ARPACK lib" FORCE)
      endif()
    endif()
    if (XLIFEPP_ARPACK_LIB STREQUAL " ")
      find_arpack(${XLIFEPP_ARPACK_LIB_DIR} ARPACK_FOUND XLIFEPP_ARPACK_LIB)
    else()
      message(STATUS "Found ARPACK : ${XLIFEPP_ARPACK_LIB}")
    endif()
    if (NOT XLIFEPP_ARPACK_LIB STREQUAL " ")
      if (XLIFEPP_FROM_SRCS)
        if(WIN32)
          set(XLIFEPP_SYSTEM_ARPACK OFF CACHE BOOL "Determine which ARPACK to use (ON, OFF)" FORCE)
        endif()
	    endif()
      if (NOT LAPACK_FOUND)
        message(STATUS "ARPACK needs LAPACK to be used ! ARPACK is not used")
      elseif (NOT BLAS_FOUND)
        message(STATUS "ARPACK needs BLAS to be used ! ARPACK is not used")
      else()
        get_filename_component(XLIFEPP_ARPACK_LIB_DIR ${XLIFEPP_ARPACK_LIB} REALPATH)
        message(STATUS "ARPACK is used")
        if (CMAKE_COMPILER_IS_GNUCXX)
          addIfEnsuredOnce(VAR GCC_COVERAGE_COMPILE_FLAGS "-fpermissive")
          addIfEnsuredOnce(VAR CMAKE_CXX_FLAGS "${GCC_COVERAGE_COMPILE_FLAGS}")
          set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}" CACHE STRING "compile flags" FORCE)
        endif()
      endif()
    else()
      set(AUTOBUILDARPACK TRUE)
      set(XLIFEPP_SYSTEM_ARPACK OFF CACHE BOOL "Determine which ARPACK to use (ON, OFF)" FORCE)
      message(STATUS "ARPACK is not found ! We will compile and use the provided one")
    endif()
  else()
    set(AUTOBUILDARPACK TRUE)
  endif()
  if (AUTOBUILDARPACK)
    set(XLIFEPP_INTERNAL_ARPACK_LIB_DIR " ")
    set(XLIFEPP_INTERNAL_ARPACK_LIB " ")
    if (XLIFEPP_FROM_SRCS)
      set(XLIFEPP_INTERNAL_ARPACK_LIB_DIR ${LIBRARY_OUTPUT_PATH})
      set(XLIFEPP_INTERNAL_ARPACK_LIB "${LIBRARY_OUTPUT_PATH}/libarpack.a")
    else()
      set(XLIFEPP_INTERNAL_ARPACK_LIB_DIR ${CMAKE_SOURCE_DIR}/lib)
      set(XLIFEPP_INTERNAL_ARPACK_LIB "${CMAKE_SOURCE_DIR}/lib/libarpack.a")
      if(EXISTS ${XLIFEPP_INTERNAL_ARPACK_LIB})
        set(XLIFEPP_ARPACK_LIB_DIR ${XLIFEPP_INTERNAL_ARPACK_LIB_DIR} CACHE PATH "Path to ARPACK lib directory" FORCE)
        set(XLIFEPP_ARPACK_LIB ${XLIFEPP_INTERNAL_ARPACK_LIB} CACHE FILEPATH "ARPACK lib" FORCE)
      endif()
    endif()
    if (NOT ${XLIFEPP_ARPACK_LIB} STREQUAL ${XLIFEPP_INTERNAL_ARPACK_LIB})
      if (XLIFEPP_FROM_SRCS)
        enable_language(Fortran)
        if (CMAKE_SYSTEM_NAME MATCHES "Darwin")
          cmake_policy(SET CMP0042 NEW) # needed by a cmake warning because of compilation of arpack library
        endif()
        set(FORTRAN_ENABLED TRUE)
        set(XLIFEPP_ENABLE_GFORTRAN TRUE)
        message(STATUS "ARPACK will be compiled and used")
        add_subdirectory(ext/ARPACK)
      else()
        message(STATUS "ARPACK is used (provided library)")
      endif()
      set(ARPACK XLIFEPP_WITH_ARPACK)
      set(ARPACK_FOUND TRUE)
      set(XLIFEPP_ARPACK_LIB_DIR ${XLIFEPP_INTERNAL_ARPACK_LIB_DIR} CACHE PATH "Path to arpack lib directory" FORCE)
      set(XLIFEPP_ARPACK_LIB ${XLIFEPP_INTERNAL_ARPACK_LIB} CACHE FILEPATH "ARPACK lib" FORCE)
      if (CMAKE_COMPILER_IS_GNUCXX)
        addIfEnsuredOnce(VAR GCC_COVERAGE_COMPILE_FLAGS "-fpermissive")
        addIfEnsuredOnce(VAR CMAKE_CXX_FLAGS "${GCC_COVERAGE_COMPILE_FLAGS}")
        set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}" CACHE STRING "compile flags" FORCE)
      endif()
    else()
      if (NOT EXISTS ${XLIFEPP_INTERNAL_ARPACK_LIB})
        if (XLIFEPP_FROM_SRCS)
          # this case shoud not occur, except if libarpack.a was compiled in another place by error or was deleted by error
          enable_language(Fortran)
          if (CMAKE_SYSTEM_NAME MATCHES "Darwin")
            cmake_policy(SET CMP0042 NEW) # needed by a cmake warning because of compilation of arpack library
          endif()
          set(FORTRAN_ENABLED TRUE)
          set(XLIFEPP_ENABLE_GFORTRAN TRUE)
          message(STATUS "ARPACK will be compiled and used")
          add_subdirectory(ext/ARPACK)
          set(ARPACK XLIFEPP_WITH_ARPACK)
          set(ARPACK_FOUND TRUE)
          set(XLIFEPP_ARPACK_LIB_DIR ${XLIFEPP_INTERNAL_ARPACK_LIB_DIR} CACHE PATH "Path to arpack lib directory" FORCE)
          set(XLIFEPP_ARPACK_LIB ${XLIFEPP_INTERNAL_ARPACK_LIB} CACHE FILEPATH "ARPACK lib" FORCE)
          if (CMAKE_COMPILER_IS_GNUCXX)
            addIfEnsuredOnce(VAR GCC_COVERAGE_COMPILE_FLAGS "-fpermissive")
            addIfEnsuredOnce(VAR CMAKE_CXX_FLAGS "${GCC_COVERAGE_COMPILE_FLAGS}")
            set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}" CACHE STRING "compile flags" FORCE)
          endif()
        else()
          message(STATUS "ARPACK is not found ! It is not used")
        endif()
      else()
        message(STATUS "ARPACK is used")
      endif()
    endif()
  endif()
else()
  message(STATUS "ARPACK is not used")
endif()
mark_as_advanced(XLIFEPP_ARPACK_LIB)

##### UMFPACK -> XLIFEPP_WITH_UMFPACK or XLIFEPP_WITHOUT_UMFPACK
##### ------------------------ LAPACK and BLAS required --------------------------
##### cache options: XLIFEPP_UMFPACK_INCLUDE_DIR, XLIFEPP_UMFPACK_LIB_DIR, XLIFEPP_ENABLE_UMFPACK
#####                XLIFEPP_SUITESPARSE_HOME_DIR, XLIFEPP_SUITESPARSE_CONFIG_DIR, XLIFEPP_SUITESPARSE_LIB_DIR
#####                XLIFEPP_AMD_INCLUDE_DIR, XLIFEPP_AMD_LIB_DIR
#####                XLIFEPP_CHOLMOD_INCLUDE_DIR, XLIFEPP_CHOLMOD_LIB_DIR
#####                XLIFEPP_CAMD_INCLUDE_DIR, XLIFEPP_CAMD_LIB_DIR
#####                XLIFEPP_COLAMD_INCLUDE_DIR, XLIFEPP_COLAMD_LIB_DIR
#####                XLIFEPP_CCOLAMD_INCLUDE_DIR, XLIFEPP_CCOLAMD_LIB_DIR
#####                XLIFEPP_METIS_INCLUDE_DIR, XLIFEPP_METIS_LIB_DIR
#####                XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR, XLIFEPP_SUITESPARSECONFIG_LIB_DIR
# Checking UmfPack package
set(XLIFEPP_SUITESPARSE_HOME_DIR ${SUITESPARSE_HOME_DIR_DEFAULT} CACHE PATH "Path to SuiteSparse home directory")
set(XLIFEPP_SUITESPARSE_CONFIG_DIR " " CACHE PATH "Path to SuiteSparse_config directory")
set(XLIFEPP_UMFPACK_INCLUDE_DIR " " CACHE PATH "Path to UMFPACK include directory")
set(XLIFEPP_UMFPACK_LIB_DIR " " CACHE PATH "Path to UMFPACK lib directory")
set(XLIFEPP_AMD_INCLUDE_DIR " " CACHE PATH "Path to AMD include directory")
set(XLIFEPP_AMD_LIB_DIR " " CACHE PATH "Path to AMD lib directory")
set(XLIFEPP_CHOLMOD_INCLUDE_DIR " " CACHE PATH "Path to CHOLMOD include directory")
set(XLIFEPP_CHOLMOD_LIB_DIR " " CACHE PATH "Path to CHOLMOD lib directory")
set(XLIFEPP_CAMD_INCLUDE_DIR " " CACHE PATH "Path to CAMD include directory")
set(XLIFEPP_CAMD_LIB_DIR " " CACHE PATH "Path to CAMD lib directory")
set(XLIFEPP_COLAMD_INCLUDE_DIR " " CACHE PATH "Path to COLAMD include directory")
set(XLIFEPP_COLAMD_LIB_DIR " " CACHE PATH "Path to COLAMD lib directory")
set(XLIFEPP_CCOLAMD_INCLUDE_DIR " " CACHE PATH "Path to CCOLAMD include directory")
set(XLIFEPP_CCOLAMD_LIB_DIR " " CACHE PATH "Path to CCOLAMD lib directory")
set(XLIFEPP_METIS_INCLUDE_DIR " " CACHE PATH "Path to METIS include directory")
set(XLIFEPP_METIS_LIB_DIR " " CACHE PATH "Path to METIS lib directory")
set(XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR " " CACHE PATH "Path to SuiteSparse_config include directory")
set(XLIFEPP_SUITESPARSECONFIG_LIB_DIR " " CACHE PATH "Path to SuiteSparse_config lib directory")
if (APPLE)
  set(XLIFEPP_SUITESPARSE_LIB_DIR " " CACHE PATH "Path to SuiteSparse lib directory (Apple only)")
endif()
set(UMFPACK XLIFEPP_WITHOUT_UMFPACK)
if (XLIFEPP_ENABLE_UMFPACK)
  set(XLIFEPP_ENABLE_METIS ON CACHE BOOL "Enable Metis (ON, OFF)" FORCE)
  if (NOT LAPACK_FOUND)
    message(STATUS "UMFPACK needs LAPACK to be used ! UMFPACK is not used")
  elseif (NOT BLAS_FOUND)
    message(STATUS "UMFPACK needs BLAS to be used ! UMFPACK is not used")
  else()
    # check if XLIFEPP_SUITESPARSE_HOME_DIR option is already in cache:
    xlifeppSetDefaultValue(SUITESPARSE_DIR XLIFEPP_SUITESPARSE_HOME_DIR ${SUITESPARSE_HOME_DIR_DEFAULT})

    # set default values for each option according to emptiness of XLIFEPP_SUITESPARSE_HOME_DIR
    set(DEFAULTVALUE_SUITESPARSECONFIG_DIR " ")
    set(DEFAULTVALUE_UMFPACK_INCLUDE_DIR " ")
    set(DEFAULTVALUE_UMFPACK_LIB_DIR " ")
    set(DEFAULTVALUE_AMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_AMD_LIB_DIR " ")
    set(DEFAULTVALUE_CHOLMOD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CHOLMOD_LIB_DIR " ")
    set(DEFAULTVALUE_CAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CAMD_LIB_DIR " ")
    set(DEFAULTVALUE_COLAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_COLAMD_LIB_DIR " ")
    set(DEFAULTVALUE_CCOLAMD_INCLUDE_DIR " ")
    set(DEFAULTVALUE_CCOLAMD_LIB_DIR " ")
    set(DEFAULTVALUE_METIS_INCLUDE_DIR " ")
    set(DEFAULTVALUE_METIS_LIB_DIR " ")
    set(DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR " ")
    set(DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR " ")
    if (APPLE)
      set(DEFAULTVALUE_SUITESPARSE_LIB_DIR " ")
    endif()
    if (NOT SUITESPARSE_DIR STREQUAL " ")
      set(DEFAULTVALUE_SUITESPARSECONFIG_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      set(DEFAULTVALUE_UMFPACK_INCLUDE_DIR ${SUITESPARSE_DIR}/UMFPACK)
      set(DEFAULTVALUE_UMFPACK_LIB_DIR ${SUITESPARSE_DIR}/UMFPACK)
      set(DEFAULTVALUE_AMD_INCLUDE_DIR ${SUITESPARSE_DIR}/AMD)
      set(DEFAULTVALUE_AMD_LIB_DIR ${SUITESPARSE_DIR}/AMD)
      set(DEFAULTVALUE_CHOLMOD_INCLUDE_DIR ${SUITESPARSE_DIR}/CHOLMOD)
      set(DEFAULTVALUE_CHOLMOD_LIB_DIR ${SUITESPARSE_DIR}/CHOLMOD)
      set(DEFAULTVALUE_CAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/CAMD)
      set(DEFAULTVALUE_CAMD_LIB_DIR ${SUITESPARSE_DIR}/CAMD)
      set(DEFAULTVALUE_COLAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/COLAMD)
      set(DEFAULTVALUE_COLAMD_LIB_DIR ${SUITESPARSE_DIR}/COLAMD)
      set(DEFAULTVALUE_CCOLAMD_INCLUDE_DIR ${SUITESPARSE_DIR}/CCOLAMD)
      set(DEFAULTVALUE_CCOLAMD_LIB_DIR ${SUITESPARSE_DIR}/CCOLAMD)
      set(DEFAULTVALUE_METIS_INCLUDE_DIR ${SUITESPARSE_DIR}/metis-5.1.0)
      set(DEFAULTVALUE_METIS_LIB_DIR ${SUITESPARSE_DIR}/metis-5.1.0)
      set(DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      set(DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR ${SUITESPARSE_DIR}/SuiteSparse_config)
      if(APPLE)
        set(DEFAULTVALUE_SUITESPARSE_LIB_DIR ${SUITESPARSE_DIR})
      endif()
    endif()

    # check if other options are already in cache:
    xlifeppSetDefaultValue(SUITESPARSECONFIG_DIR XLIFEPP_SUITESPARSE_CONFIG_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_DIR})
    xlifeppSetDefaultValue(UMFPACK_INCLUDE_DIR XLIFEPP_UMFPACK_INCLUDE_DIR ${DEFAULTVALUE_UMFPACK_INCLUDE_DIR})
    xlifeppSetDefaultValue(UMFPACK_LIB_DIR XLIFEPP_UMFPACK_LIB_DIR ${DEFAULTVALUE_UMFPACK_LIB_DIR})
    xlifeppSetDefaultValue(AMD_INCLUDE_DIR XLIFEPP_AMD_INCLUDE_DIR ${DEFAULTVALUE_AMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(AMD_LIB_DIR XLIFEPP_AMD_LIB_DIR ${DEFAULTVALUE_AMD_LIB_DIR})
    xlifeppSetDefaultValue(CHOLMOD_INCLUDE_DIR XLIFEPP_CHOLMOD_INCLUDE_DIR ${DEFAULTVALUE_CHOLMOD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CHOLMOD_LIB_DIR XLIFEPP_CHOLMOD_LIB_DIR ${DEFAULTVALUE_CHOLMOD_LIB_DIR})
    xlifeppSetDefaultValue(CAMD_INCLUDE_DIR XLIFEPP_CAMD_INCLUDE_DIR ${DEFAULTVALUE_CAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CAMD_LIB_DIR XLIFEPP_CAMD_LIB_DIR ${DEFAULTVALUE_CAMD_LIB_DIR})
    xlifeppSetDefaultValue(COLAMD_INCLUDE_DIR XLIFEPP_COLAMD_INCLUDE_DIR ${DEFAULTVALUE_COLAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(COLAMD_LIB_DIR XLIFEPP_COLAMD_LIB_DIR ${DEFAULTVALUE_COLAMD_LIB_DIR})
    xlifeppSetDefaultValue(CCOLAMD_INCLUDE_DIR XLIFEPP_CCOLAMD_INCLUDE_DIR ${DEFAULTVALUE_CCOLAMD_INCLUDE_DIR})
    xlifeppSetDefaultValue(CCOLAMD_LIB_DIR XLIFEPP_CCOLAMD_LIB_DIR ${DEFAULTVALUE_CCOLAMD_LIB_DIR})
    xlifeppSetDefaultValue(METIS_INCLUDE_DIR XLIFEPP_METIS_INCLUDE_DIR ${DEFAULTVALUE_METIS_INCLUDE_DIR})
    xlifeppSetDefaultValue(METIS_LIB_DIR XLIFEPP_METIS_LIB_DIR ${DEFAULTVALUE_METIS_LIB_DIR})
    xlifeppSetDefaultValue(SUITESPARSECONFIG_INCLUDE_DIR XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_INCLUDE_DIR})
    xlifeppSetDefaultValue(SUITESPARSECONFIG_LIB_DIR XLIFEPP_SUITESPARSECONFIG_LIB_DIR ${DEFAULTVALUE_SUITESPARSECONFIG_LIB_DIR})
    if (APPLE)
      xlifeppSetDefaultValue(SUITESPARSE_LIB_DIR XLIFEPP_SUITESPARSE_LIB_DIR ${DEFAULTVALUE_SUITESPARSE_LIB_DIR})
    endif()
    find_package(UMFPACK)
    if (UMFPACK_FOUND)
      include_directories(${UMFPACK_INCLUDES})
      set(UMFPACK XLIFEPP_WITH_UMFPACK)
      set(XLIFEPP_ENABLE_UMFPACK TRUE)
      set(XLIFEPP_UMFPACK_INCLUDE_DIR ${UMFPACK_INCLUDES})
      if (UMFPACK_LIBRARY)
        get_filename_component(XLIFEPP_UMFPACK_LIB_DIR ${UMFPACK_LIBRARY} PATH)
      endif()
      set(XLIFEPP_AMD_INCLUDE_DIR ${AMD_INCLUDE})
      if (AMD_LIBRARY)
        get_filename_component(XLIFEPP_AMD_LIB_DIR ${AMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CHOLMOD_INCLUDE_DIR ${CHOLMOD_INCLUDE})
      if (CHOLMOD_LIBRARY)
        get_filename_component(XLIFEPP_CHOLMOD_LIB_DIR ${CHOLMOD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CAMD_INCLUDE_DIR ${CAMD_INCLUDE})
      if (CAMD_LIBRARY)
        get_filename_component(XLIFEPP_CAMD_LIB_DIR ${CAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_COLAMD_INCLUDE_DIR ${COLAMD_INCLUDE})
      if (COLAMD_LIBRARY)
        get_filename_component(XLIFEPP_COLAMD_LIB_DIR ${COLAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_CCOLAMD_INCLUDE_DIR ${CCOLAMD_INCLUDE})
      if (CCOLAMD_LIBRARY)
        get_filename_component(XLIFEPP_CCOLAMD_LIB_DIR ${CCOLAMD_LIBRARY} PATH)
      endif()
      set(XLIFEPP_METIS_INCLUDE_DIR ${METIS_INCLUDE})
      if (METIS_LIBRARY)
        get_filename_component(XLIFEPP_METIS_LIB_DIR ${METIS_LIBRARY} PATH)
      endif()
      set(XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR ${SUITESPARSECONFIG_INCLUDE})
      if (SUITESPARSECONFIG_LIBRARY)
        get_filename_component(XLIFEPP_SUITESPARSECONFIG_LIB_DIR ${SUITESPARSECONFIG_LIBRARY} PATH)
      endif()
      if (APPLE)
        if (SUITESPARSE_LIBRARY)
          get_filename_component(XLIFEPP_SUITESPARSE_LIB_DIR ${SUITESPARSE_LIBRARY} PATH)
        endif()
      endif()
      message(STATUS "UMFPACK is used")
    else()
      message(STATUS "UMFPACK is not found ! It is not used")
    endif()
  endif()
else()
  message(STATUS "UMFPACK is not used")
endif()

mark_as_advanced(XLIFEPP_SUITESPARSE_CONFIG_DIR
                 XLIFEPP_AMD_INCLUDE_DIR XLIFEPP_AMD_LIB_DIR
                 XLIFEPP_CHOLMOD_INCLUDE_DIR XLIFEPP_CHOLMOD_LIB_DIR
                 XLIFEPP_CAMD_INCLUDE_DIR XLIFEPP_CAMD_LIB_DIR
                 XLIFEPP_COLAMD_INCLUDE_DIR XLIFEPP_COLAMD_LIB_DIR
                 XLIFEPP_CCOLAMD_INCLUDE_DIR XLIFEPP_CCOLAMD_LIB_DIR
                 XLIFEPP_METIS_INCLUDE_DIR XLIFEPP_METIS_LIB_DIR
                 XLIFEPP_SUITESPARSECONFIG_INCLUDE_DIR XLIFEPP_SUITESPARSECONFIG_LIB_DIR)
if (APPLE)
  mark_as_advanced(XLIFEPP_SUITESPARSE_LIB_DIR)
endif()

##### GMSH -> XLIFEPP_WITH_GMSH or XLIFEPP_WITHOUT_GMSH
# checking gmsh executable in PATH environment variable
set(GMSH_ADDITIONAL_PATHS "")
if (APPLE)
  set(GMSH_ADDITIONAL_PATHS /Applications/Gmsh.app/Contents/MacOS/)
endif()
if (WIN32)
  list(APPEND GMSH_ADDITIONAL_PATHS "C:/Program Files/gmsh")
  list(APPEND GMSH_ADDITIONAL_PATHS "C:/Program Files (x86)/gmsh")
endif()
if (NOT XLIFEPP_GMSH_EXECUTABLE)
  find_program(XLIFEPP_GMSH_EXECUTABLE gmsh PATHS $ENV{PATH} ${GMSH_ADDITIONAL_PATHS} DOC "Path to Gmsh executable" )
endif()
set(GMSH XLIFEPP_WITHOUT_GMSH)
if (${XLIFEPP_GMSH_EXECUTABLE} STREQUAL XLIFEPP_GMSH_EXECUTABLE_NOTFOUND OR ${XLIFEPP_GMSH_EXECUTABLE} STREQUAL XLIFEPP_GMSH_EXECUTABLE-NOTFOUND)
  message(STATUS "GMSH is not found ! You should set the XLIFEPP_GMSH_EXECUTABLE to fix this")
else()
  string(STRIP ${XLIFEPP_GMSH_EXECUTABLE} XLIFEPP_GMSH_EXECUTABLE)
  get_filename_component(XLIFEPP_GMSH_EXECUTABLE ${XLIFEPP_GMSH_EXECUTABLE} REALPATH)
  get_filename_component(XLIFEPP_GMSH_DIR ${XLIFEPP_GMSH_EXECUTABLE} DIRECTORY)
  if (XLIFEPP_GMSH_DIR)
    set(GMSH XLIFEPP_WITH_GMSH)
    execute_process(COMMAND ${XLIFEPP_GMSH_EXECUTABLE} --version OUTPUT_VARIABLE XLIFEPP_GMSH_VERSION ERROR_VARIABLE XLIFEPP_GMSH_VERSION)
    string(STRIP ${XLIFEPP_GMSH_VERSION} XLIFEPP_GMSH_VERSION)
    # in case there is more than just the version number (such as warnings)
    # we consider that there can be some text before the version number, but not after
    string(REGEX MATCHALL "[0-9]+(\\.[0-9]+)?\\.[0-9]+" GMSH_VERSION_LIST ${XLIFEPP_GMSH_VERSION} )
    list(REVERSE GMSH_VERSION_LIST)
    list(GET GMSH_VERSION_LIST 0 XLIFEPP_GMSH_VERSION)
    if (XLIFEPP_GMSH_VERSION)
      if (${XLIFEPP_GMSH_VERSION} VERSION_GREATER_EQUAL 2.8.5)
        message(STATUS "Found GMSH : ${XLIFEPP_GMSH_EXECUTABLE} ${XLIFEPP_GMSH_VERSION}")
      else()
        set(GMSH XLIFEPP_WITHOUT_GMSH)
        message(STATUS "Found GMSH is too old ! Version ${XLIFEPP_GMSH_VERSION} must be greater than 2.8.5 ! GMSH is not used")
      endif()
    else()
      set(GMSH XLIFEPP_WITHOUT_GMSH)
      message(STATUS "GMSH version cannot be determined ! Its call gives an execution error ! GMSH is not used")
    endif()
  else()
    message(STATUS "GMSH is not used")
  endif()
endif()

##### OPENCASCADE -> XLIFEPP_WITH_OPENCASCADE, XLIFEPP_WITHOUT_OPENCASCADE
set(OPENCASCADE XLIFEPP_WITHOUT_OPENCASCADE)
if (XLIFEPP_ENABLE_OPENCASCADE)
  if (NOT XLIFEPP_OPENCASCADE_LIBRARIES AND NOT XLIFEPP_OPENCASCADE_INCLUDE_DIR)
    find_package(XLIFEPP_OPENCASCADE)
    if (XLIFEPP_OPENCASCADE_FOUND)
      message(STATUS "Found OpenCASCADE : ${XLIFEPP_OPENCASCADE_DIR}")
      set(OPENCASCADE XLIFEPP_WITH_OPENCASCADE)
      include_directories(${XLIFEPP_OPENCASCADE_INCLUDE_DIR})
      foreach (occlib ${XLIFEPP_OPENCASCADE_LIBRARIES})
        # message(STATUS "${occlib}")
        list(APPEND libexts ${occlib})
      endforeach()
    else()
      message(STATUS "OpenCASCADE is not found ! It is not used")
    endif()
  else()
    message(STATUS "OpenCASCADE is used")
  endif()
else()
  message(STATUS "OpenCASCADE is not used")
endif()

##### PARAVIEW -> XLIFEPP_WITH_PARAVIEW or XLIFEPP_WITHOUT_PARAVIEW
# checking paraview executable in PATH environment variable
set(PARAVIEW_ADDITIONAL_PATHS  "")
if (APPLE)
  set(PARAVIEW_ADDITIONAL_PATHS /Applications/paraview.app/Contents/MacOS/)
endif()
if (WIN32)
  list(APPEND PARAVIEW_ADDITIONAL_PATHS "C:/Program Files/ParaView/bin")
  list(APPEND PARAVIEW_ADDITIONAL_PATHS "C:/Program Files (x86)/ParaView/bin")
endif()
set(PARAVIEW XLIFEPP_WITHOUT_PARAVIEW)
if (NOT XLIFEPP_PARAVIEW_EXECUTABLE)
  find_program(XLIFEPP_PARAVIEW_EXECUTABLE paraview PATHS $ENV{PATH} ${PARAVIEW_ADDITIONAL_PATHS} DOC "Path to Paraview executable")
endif()
if (${XLIFEPP_PARAVIEW_EXECUTABLE} STREQUAL XLIFEPP_PARAVIEW_EXECUTABLE_NOTFOUND OR ${XLIFEPP_PARAVIEW_EXECUTABLE} STREQUAL XLIFEPP_PARAVIEW_EXECUTABLE-NOTFOUND)
  message(STATUS "Paraview is not found ! It is not used")
else()
  string(STRIP ${XLIFEPP_PARAVIEW_EXECUTABLE} XLIFEPP_PARAVIEW_EXECUTABLE)
  get_filename_component(XLIFEPP_PARAVIEW_EXECUTABLE ${XLIFEPP_PARAVIEW_EXECUTABLE} REALPATH)
  get_filename_component( XLIFEPP_PARAVIEW_DIR ${XLIFEPP_PARAVIEW_EXECUTABLE} DIRECTORY)
  if (XLIFEPP_PARAVIEW_DIR)
    set(PARAVIEW XLIFEPP_WITH_PARAVIEW)
    message(STATUS "Found Paraview : " ${XLIFEPP_PARAVIEW_EXECUTABLE})
  else()
    message(STATUS "Paraview is not used")
  endif()
endif()

##### EIGEN -> XLIFEPP_WITH_EIGEN or XLIFEPP_WITHOUT_EIGEN
##### cache option: XLIFEPP_EIGEN_DIR
set(XLIFEPP_EIGEN_DIR ${CMAKE_SOURCE_DIR}/ext/Eigen/Eigen CACHE PATH "Path to Eigen directory")
set(EIGEN XLIFEPP_WITH_EIGEN)
if (${OMP} STREQUAL XLIFEPP_WITHOUT_OMP)
  message(STATUS "Eigen needs OpenMP to be activated ! It is not used")
  set(XLIFEPP_ENABLE_EIGEN OFF)
endif()
if (XLIFEPP_ENABLE_EIGEN)
  if (${XLIFEPP_CXX_COMPILER} STREQUAL "g++-4.7")
    set(EIGEN XLIFEPP_WITHOUT_EIGEN)
    message(STATUS "g++ 4.7 is not able to compile Eigen library ! Eigen is not used")
  else()
    include_directories(${XLIFEPP_EIGEN_DIR})
    message(STATUS "Eigen is used")
  endif()
else()
  set(EIGEN XLIFEPP_WITHOUT_EIGEN)
  message(STATUS "Eigen is not used")
endif()

##### AMOS -> XLIFEPP_WITH_AMOS or XLIFEPP_WITHOUT_AMOS
##### cache options: XLIFEPP_AMOS_DIR, XLIFEPP_AMOS_INCLUDE_DIR, XLIFEPP_AMOS_LIB_DIR, XLIFEPP_AMOS_LIB, XLIFEPP_ENABLE_AMOS
set(XLIFEPP_AMOS_DIR ${AMOS_DIR_DEFAULT} CACHE PATH "Path to Amos directory")
set(AMOS XLIFEPP_WITH_AMOS)
if (XLIFEPP_ENABLE_AMOS)
  if (XLIFEPP_FROM_SRCS)
    if (NOT EXISTS ${XLIFEPP_AMOS_LIB})
      if (NOT FORTRAN_ENABLED)
        enable_language(Fortran)
        if (CMAKE_SYSTEM_NAME MATCHES "Darwin")
          cmake_policy(SET CMP0042 NEW) # needed by a cmake warning because of compilation of arpack library
        endif()
      endif()
      set(FORTRAN_ENABLED TRUE)
      file(GLOB AMOS_SRCS ${XLIFEPP_AMOS_DIR}/*.f)
      add_library(amos STATIC ${AMOS_SRCS})
      set(XLIFEPP_AMOS_LIB_DIR ${LIBRARY_OUTPUT_PATH} CACHE PATH "Path to amos lib directory" FORCE)
      set(XLIFEPP_AMOS_LIB ${LIBRARY_OUTPUT_PATH}/libamos.a CACHE FILEPATH "Amos lib" FORCE)
      message(STATUS "Amos will be compiled and used")
    else()
      message(STATUS "Amos is used")
    endif()
  else()
    message(STATUS "Amos is used")
  endif()
  set(XLIFEPP_ENABLE_GFORTRAN TRUE)
else()
  set(AMOS XLIFEPP_WITHOUT_AMOS)
  message(STATUS "Amos is not used")
endif()

##### MAGMA -> XLIFEPP_WITH_MAGMA, XLIFEPP_WITHOUT_MAGMA
##### cache option: XLIFEPP_ENABLE_MAGMA
set(MAGMA XLIFEPP_WITHOUT_MAGMA)
if (XLIFEPP_ENABLE_MAGMA)
  find_package(MAGMA)
  if (MAGMA_FOUND)
    include_directories(${MAGMA_INCLUDE})
    set(MAGMA XLIFEPP_WITH_MAGMA)
    set(XLIFEPP_MAGMA_DIR ${MAGMA_INCLUDE})
    message(STATUS "Magma is used")
  else()
    message(STATUS "Magma is not found ! It is not used")
  endif()
else()
  message(STATUS "Magma is not used")
endif()

##### METIS -> XLIFEPP_WITH_METIS, XLIFEPP_WITHOUT_METIS
##### cache options: XLIFEPP_METIS_INCLUDE_DIR, XLIFEPP_METIS_LIB_DIR, XLIFEPP_ENABLE_METIS
set(METIS XLIFEPP_WITHOUT_METIS)
if (XLIFEPP_ENABLE_UMFPACK AND METIS_FOUND)
  message(STATUS "Metis is used (through UMFPACK)")
  set(XLIFEPP_METIS_DIR ${METIS_INCLUDE})
  set(XLIFEPP_METIS_LIB ${METIS_LIBRARY})
  set(METIS XLIFEPP_WITH_METIS)
elseif(XLIFEPP_ENABLE_METIS)
  find_package(METIS)
  if (METIS_FOUND)
    include_directories(${METIS_INCLUDE})
    set(XLIFEPP_METIS_DIR ${METIS_INCLUDE})
    set(XLIFEPP_METIS_LIB ${METIS_LIBRARY})
    set(METIS XLIFEPP_WITH_METIS)
    message(STATUS "Metis is used")
  else()
    message(STATUS "Metis is not found ! It is not used")
  endif()
else()
  message(STATUS "Metis is not used")
endif()

##### MPI -> XLIFEPP_WITH_MPI, XLIFEPP_WITHOUT_MPI
set(MPI XLIFEPP_WITHOUT_MPI)
if (XLIFEPP_ENABLE_MPI)
  find_package(MPI)
  if (MPI_CXX_FOUND)
    set(MPI XLIFEPP_WITH_MPI)
    include_directories(SYSTEM ${MPI_INCLUDE_PATH})
    set(XLIFEPP_MPI_INCLUDE_DIR ${MPI_CXX_INCLUDE_PATH} CACHE PATH "Path to MPI include directory")
    set(XLIFEPP_MPI_LIB ${MPI_CXX_LIBRARIES} CACHE PATH "Path to MPI libraries")
    get_filename_component(XLIFEPP_MPI_LIB_DIR ${XLIFEPP_MPI_LIB} DIRECTORY)
    message(STATUS "MPI is used")
  else()
    message(STATUS "MPI is not found ! It is not used")
  endif()
else()
  message(STATUS "MPI is not used")
endif()

###### post work ######

# append arpack library to the libexts list
# Lapack and blas are already appended
if (XLIFEPP_ENABLE_ARPACK)
  foreach(lib ${XLIFEPP_ARPACK_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append umfpack library to the libexts list
# Lapack and blas are already appended
if (XLIFEPP_ENABLE_UMFPACK)
  foreach (lib ${UMFPACK_LIBRARIES})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append lapack library to the libexts list
if (XLIFEPP_ENABLE_LAPACK)
  foreach (lib ${XLIFEPP_LAPACK_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append blas library to the libexts list
if (XLIFEPP_ENABLE_BLAS)
  foreach (lib ${XLIFEPP_BLAS_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append amos library to the libexts list
if (XLIFEPP_ENABLE_AMOS)
  foreach(lib ${XLIFEPP_AMOS_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append magma library to the libexts list
if (XLIFEPP_ENABLE_MAGMA)
  foreach (lib ${XLIFEPP_MAGMA_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append magma library to the libexts list
if (XLIFEPP_ENABLE_METIS AND NOT XLIFEPP_ENABLE_UMFPACK)
  foreach (lib ${XLIFEPP_METIS_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

# append mpi library to the libexts list
if (XLIFEPP_ENABLE_MPI)
  foreach (lib ${XLIFEPP_MPI_LIB})
    get_filename_component(realLib ${lib} REALPATH)
    list(APPEND libexts ${realLib})
  endforeach()
endif()

set(XLIFEPP_FORTRAN_LIB_DIR " " CACHE PATH "Path to FORTRAN lib directory")
set(XLIFEPP_FORTRAN_LIB " " CACHE FILEPATH "FORTRAN lib with full path")

if (XLIFEPP_ENABLE_GFORTRAN)
  if (NOT ${XLIFEPP_FORTRAN_LIB} STREQUAL " ")
    if (NOT EXISTS ${XLIFEPP_FORTRAN_LIB})
    set(XLIFEPP_FORTRAN_LIB " " CACHE FILEPATH "FORTRAN lib with full path" FORCE)
    endif()
  endif()
  if (${XLIFEPP_FORTRAN_LIB} STREQUAL " ")
    if (NOT ${XLIFEPP_FORTRAN_LIB_DIR} STREQUAL " ")
      if (NOT EXISTS ${XLIFEPP_FORTRAN_LIB_DIR})
        set(XLIFEPP_FORTRAN_LIB_DIR " " CACHE PATH "Path to FORTRAN lib directory")
      endif()
    endif()
    addIfEnsuredOnce(LIST libexts "-lgfortran")
    # on Apple computers with clang++, we check additionnal paths from homebrew package manager
    if (${XLIFEPP_FORTRAN_LIB_DIR} STREQUAL " ")
      if (APPLE)
        get_filename_component(COMPILER_EXE ${XLIFEPP_CXX_REAL_COMPILER} NAME)
        if (${COMPILER_EXE} STREQUAL "clang++")
          file(GLOB_RECURSE FORTRAN_LIBS /usr/local/Cellar/gcc*/*/libgfortran*)
          foreach (FORTRAN_LIB ${FORTRAN_LIBS})
            get_filename_component(FORTRAN_LIB_DIR ${FORTRAN_LIB} PATH)
            list(APPEND FORTRAN_LIB_DIRS ${FORTRAN_LIB_DIR})
          endforeach()
          list(LENGTH FORTRAN_LIB_DIRS FORTRAN_NB_LIB_DIRS)
          if (NOT ${FORTRAN_NB_LIB_DIRS} STREQUAL "0")
            list(SORT FORTRAN_LIB_DIRS)
            list(REMOVE_DUPLICATES FORTRAN_LIB_DIRS)
            list(GET FORTRAN_LIB_DIRS 0 XLIFEPP_FORTRAN_LIB_DIR)
            set(XLIFEPP_FORTRAN_LIB_DIR CACHE PATH "Path to FORTRAN lib directory" FORCE)
          else()
            message(WARNING "No FORTRAN library found ! Please use option XLIFEPP_FORTRAN_LIB_DIR for additional search directory")
          endif()
        else()
          message(STATUS "${XLIFEPP_CXX_COMPILER} is supposed to know where to find FORTRAN library")
        endif()
      endif()
    endif()
    if (NOT ${XLIFEPP_FORTRAN_LIB_DIR} STREQUAL " ")
      string(STRIP ${XLIFEPP_FORTRAN_LIB_DIR} XLIFEPP_FORTRAN_LIB_DIR)
      get_filename_component(XLIFEPP_FORTRAN_LIB_DIR ${XLIFEPP_FORTRAN_LIB_DIR} REALPATH)
      list(APPEND libexts "-L${XLIFEPP_FORTRAN_LIB_DIR}")
      message(STATUS "FORTRAN library found : ${XLIFEPP_FORTRAN_LIB_DIR}")
    endif()
  else()
    message(STATUS "FORTRAN library used : ${XLIFEPP_FORTRAN_LIB}")
    string(STRIP ${XLIFEPP_FORTRAN_LIB} XLIFEPP_FORTRAN_LIB)
    list(APPEND libexts "${XLIFEPP_FORTRAN_LIB}")
  endif()
endif()
